; Installer script for win32 Deluge
; Original Author: Herman Bloggs <hermanator12002@yahoo.com>
; Updated By: Daniel Atallah <daniel_atallah@yahoo.com>
; Adapted for deluge by Aurlien Pocheville <slurdge at slurdge dot org>

; NOTE: this .NSI script is intended for NSIS 2.27
;

!include "version.nsh"
!include "fileassoc.nsh"

;--------------------------------
;Global Variables
Var name
Var ISSILENT
Var STARTUP_RUN_KEY

;--------------------------------
;Configuration

;The name var is set in .onInit
Name $name

!ifdef DEBUG
OutFile "deluge-${DELUGE_VERSION}-debug.exe"
!else
OutFile "deluge-${DELUGE_VERSION}.exe"
!endif

SetCompressor /SOLID lzma

ShowInstDetails show
ShowUninstDetails show
SetDateSave on

; $name and $INSTDIR are set in .onInit function..

!include "MUI.nsh"
!include "Sections.nsh"
!include "WinVer.nsh"
!include "LogicLib.nsh"

!include "FileFunc.nsh"
!insertmacro GetParameters
!insertmacro GetOptions
!insertmacro GetParent

;--------------------------------
;Defines

!define DELUGE_NSIS_INCLUDE_PATH		"."

!define DELUGE_REG_KEY				"SOFTWARE\deluge"
!define DELUGE_UNINSTALL_KEY			"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\Deluge"

!define HKLM_APP_PATHS_KEY			"SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\deluge.exe"
!define STARTUP_RUN_KEY				"SOFTWARE\Microsoft\Windows\CurrentVersion\Run"
!define DELUGE_UNINST_EXE			"deluge-uninst.exe"

;--------------------------------
;Version resource
VIProductVersion "${DELUGE_VERSION}"
VIAddVersionKey "ProductName" "Deluge"
VIAddVersionKey "FileVersion" "${DELUGE_VERSION}"
VIAddVersionKey "ProductVersion" "${DELUGE_VERSION}"
VIAddVersionKey "LegalCopyright" ""
VIAddVersionKey "FileDescription" "Deluge Installer"

;--------------------------------
;Reserve files used in .onInit
;for faster start-up
ReserveFile "${NSISDIR}\Plugins\System.dll"
!insertmacro MUI_RESERVEFILE_INSTALLOPTIONS
!insertmacro MUI_RESERVEFILE_LANGDLL

;--------------------------------
;Modern UI Configuration

  !define MUI_ICON				"..\pixmaps\logo.ico"
  !define MUI_UNICON				"..\pixmaps\logo.ico"
  ;!define MUI_WELCOMEFINISHPAGE_BITMAP		".\pixmaps\deluge-intro.bmp"
  !define MUI_HEADERIMAGE
  !define MUI_HEADERIMAGE_BITMAP		"deluge-about.bmp"

  ; Alter License section
  !define MUI_LICENSEPAGE_BUTTON		$(DELUGE_LICENSE_BUTTON)
  !define MUI_LICENSEPAGE_TEXT_BOTTOM		$(DELUGE_LICENSE_BOTTOM_TEXT)

  !define MUI_LANGDLL_REGISTRY_ROOT "HKCU"
  !define MUI_LANGDLL_REGISTRY_KEY ${DELUGE_REG_KEY}
  !define MUI_LANGDLL_REGISTRY_VALUENAME "Installer Language"

  !define MUI_COMPONENTSPAGE_SMALLDESC
  !define MUI_ABORTWARNING

  ;Finish Page config
  !define MUI_FINISHPAGE_NOAUTOCLOSE
  !define MUI_FINISHPAGE_RUN			"$INSTDIR\deluge.exe"
  !define MUI_FINISHPAGE_RUN_NOTCHECKED
  ;!define MUI_FINISHPAGE_LINK			$(DELUGE_FINISH_VISIT_WEB_SITE)
  ;!define MUI_FINISHPAGE_LINK_LOCATION		"http://deluge.im"

;--------------------------------
;Pages

  ;!define MUI_PAGE_CUSTOMFUNCTION_PRE		preWelcomePage
  !insertmacro MUI_PAGE_WELCOME
  !insertmacro MUI_PAGE_LICENSE			"../LICENSE"
  !insertmacro MUI_PAGE_COMPONENTS

  ; Deluge install dir page
  !insertmacro MUI_PAGE_DIRECTORY

  !insertmacro MUI_PAGE_INSTFILES
  !insertmacro MUI_PAGE_FINISH

  !insertmacro MUI_UNPAGE_WELCOME
  !insertmacro MUI_UNPAGE_CONFIRM
  !insertmacro MUI_UNPAGE_INSTFILES
  !insertmacro MUI_UNPAGE_FINISH

;--------------------------------
;Languages

  ;; English goes first because its the default. The rest are
  ;; in alphabetical order (at least the strings actually displayed
  ;; will be).

  !insertmacro MUI_LANGUAGE "English"

  !insertmacro MUI_LANGUAGE "Afrikaans"
  !insertmacro MUI_LANGUAGE "Albanian"
  !insertmacro MUI_LANGUAGE "Bulgarian"
  !insertmacro MUI_LANGUAGE "Catalan"
  !insertmacro MUI_LANGUAGE "Czech"
  !insertmacro MUI_LANGUAGE "Danish"
  !insertmacro MUI_LANGUAGE "SimpChinese"
  !insertmacro MUI_LANGUAGE "TradChinese"
  !insertmacro MUI_LANGUAGE "German"
  !insertmacro MUI_LANGUAGE "Spanish"
  !insertmacro MUI_LANGUAGE "Farsi"
  !insertmacro MUI_LANGUAGE "Finnish"
  !insertmacro MUI_LANGUAGE "French"
  !insertmacro MUI_LANGUAGE "Hebrew"
  !insertmacro MUI_LANGUAGE "Italian"
  !insertmacro MUI_LANGUAGE "Japanese"
  !insertmacro MUI_LANGUAGE "Korean"
  !insertmacro MUI_LANGUAGE "Kurdish"
  !insertmacro MUI_LANGUAGE "Lithuanian"
  !insertmacro MUI_LANGUAGE "Hungarian"
  !insertmacro MUI_LANGUAGE "Dutch"
  !insertmacro MUI_LANGUAGE "Norwegian"
  !insertmacro MUI_LANGUAGE "Polish"
  !insertmacro MUI_LANGUAGE "PortugueseBR"
  !insertmacro MUI_LANGUAGE "Portuguese"
  !insertmacro MUI_LANGUAGE "Romanian"
  !insertmacro MUI_LANGUAGE "Russian"
  !insertmacro MUI_LANGUAGE "Serbian"
  !insertmacro MUI_LANGUAGE "Slovak"
  !insertmacro MUI_LANGUAGE "Slovenian"
  !insertmacro MUI_LANGUAGE "Swedish"

;--------------------------------
;Translations

  !define DELUGE_DEFAULT_LANGFILE "${DELUGE_NSIS_INCLUDE_PATH}\translations\english.nsh"

  !include "${DELUGE_NSIS_INCLUDE_PATH}\langmacros.nsh"

  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "AFRIKAANS"	"${DELUGE_NSIS_INCLUDE_PATH}\translations\afrikaans.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "ALBANIAN"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\albanian.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "BULGARIAN"	"${DELUGE_NSIS_INCLUDE_PATH}\translations\bulgarian.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "CATALAN"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\catalan.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "CZECH"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\czech.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "DANISH"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\danish.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "DUTCH"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\dutch.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "ENGLISH"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\english.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "FARSI"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\persian.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "FINNISH"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\finnish.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "FRENCH"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\french.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "GERMAN"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\german.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "HEBREW"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\hebrew.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "HUNGARIAN"	"${DELUGE_NSIS_INCLUDE_PATH}\translations\hungarian.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "ITALIAN"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\italian.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "JAPANESE"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\japanese.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "KOREAN"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\korean.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "KURDISH"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\kurdish.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "LITHUANIAN"	"${DELUGE_NSIS_INCLUDE_PATH}\translations\lithuanian.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "NORWEGIAN"	"${DELUGE_NSIS_INCLUDE_PATH}\translations\norwegian.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "POLISH"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\polish.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "PORTUGUESE"	"${DELUGE_NSIS_INCLUDE_PATH}\translations\portuguese.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "PORTUGUESEBR"	"${DELUGE_NSIS_INCLUDE_PATH}\translations\portuguese-br.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "ROMANIAN"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\romanian.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "RUSSIAN"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\russian.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "SERBIAN"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\serbian-latin.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "SIMPCHINESE"	"${DELUGE_NSIS_INCLUDE_PATH}\translations\simp-chinese.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "SLOVAK"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\slovak.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "SLOVENIAN"	"${DELUGE_NSIS_INCLUDE_PATH}\translations\slovenian.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "SPANISH"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\spanish.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "SWEDISH"		"${DELUGE_NSIS_INCLUDE_PATH}\translations\swedish.nsh"
  !insertmacro DELUGE_MACRO_INCLUDE_LANGFILE "TRADCHINESE"	"${DELUGE_NSIS_INCLUDE_PATH}\translations\trad-chinese.nsh"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Start Install Sections ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;--------------------------------
;Uninstall any old version of Deluge

Section -SecUninstallOldDeluge
  ; Check install rights..
  Call CheckUserInstallRights
  Pop $R0

  ;First try to uninstall Deluge
  StrCpy $R4 ${DELUGE_REG_KEY}
  StrCpy $R5 ${DELUGE_UNINSTALL_KEY}
  StrCpy $R6 ${DELUGE_UNINST_EXE}
  StrCpy $R7 "Deluge"

  start_comparison:
  ;If deluge is currently set to run on startup,
  ;  save the section of the Registry where the setting is before uninstalling,
  ;  so we can put it back after installing the new version
  ClearErrors
  ReadRegStr $STARTUP_RUN_KEY HKCU "${STARTUP_RUN_KEY}" $R7
  IfErrors +3
  StrCpy $STARTUP_RUN_KEY "HKCU"
  Goto +5
  ClearErrors
  ReadRegStr $STARTUP_RUN_KEY HKLM "${STARTUP_RUN_KEY}" $R7
  IfErrors +2
  StrCpy $STARTUP_RUN_KEY "HKLM"

  StrCmp $R0 "HKLM" compare_hklm
  StrCmp $R0 "HKCU" compare_hkcu done

  compare_hkcu:
      ReadRegStr $R1 HKCU $R4 ""
      ReadRegStr $R2 HKCU $R4 "Version"
      ReadRegStr $R3 HKCU "$R5" "UninstallString"
      Goto try_uninstall

  compare_hklm:
      ReadRegStr $R1 HKLM $R4 ""
      ReadRegStr $R2 HKLM $R4 "Version"
      ReadRegStr $R3 HKLM "$R5" "UninstallString"

  ; If a previous version exists, remove it
  try_uninstall:
    StrCmp $R1 "" no_version_found
      ; Version key started with 0.60a3. Prior versions can't be
      ; automatically uninstalled.
      StrCmp $R2 "" uninstall_problem
        ; Check if we have uninstall string..
        IfFileExists $R3 0 uninstall_problem
          ; Have uninstall string, go ahead and uninstall.
          SetOverwrite on
          ; Need to copy uninstaller outside of the install dir
          ClearErrors
          CopyFiles /SILENT $R3 "$TEMP\$R6"
          SetOverwrite off
          IfErrors uninstall_problem
            ; Ready to uninstall..
            ClearErrors
            ExecWait '"$TEMP\$R6" /S _?=$R1'
            IfErrors exec_error
              Delete "$TEMP\$R6"
            Goto done

            exec_error:
              Delete "$TEMP\$R6"
              Goto uninstall_problem

        no_version_found:
          Goto done

        uninstall_problem:
          ; We can't uninstall.  Either the user must manually uninstall or we ignore and reinstall over it.
          MessageBox MB_OKCANCEL $(DELUGE_PROMPT_CONTINUE_WITHOUT_UNINSTALL) /SD IDOK IDOK done
          Quit
  done:
SectionEnd

;--------------------------------
;Deluge Install Section

Section $(DELUGE_SECTION_TITLE) SecDeluge
  SectionIn 1 RO

  ; Check install rights..
  Call CheckUserInstallRights
  Pop $R0

  StrCmp $R0 "NONE" deluge_none
  StrCmp $R0 "HKLM" deluge_hklm deluge_hkcu

  deluge_hklm:
    WriteRegStr HKLM "${HKLM_APP_PATHS_KEY}" "" "$INSTDIR\deluge.exe"
    WriteRegStr HKLM "${HKLM_APP_PATHS_KEY}" "Path" "$R1\bin"
    WriteRegStr HKLM ${DELUGE_REG_KEY} "" "$INSTDIR"
    WriteRegStr HKLM ${DELUGE_REG_KEY} "Version" "${DELUGE_VERSION}"
    WriteRegStr HKLM "${DELUGE_UNINSTALL_KEY}" "DisplayName" "Deluge"
    WriteRegStr HKLM "${DELUGE_UNINSTALL_KEY}" "DisplayVersion" "${DELUGE_VERSION}"
    WriteRegStr HKLM "${DELUGE_UNINSTALL_KEY}" "HelpLink" "http://deluge-torrent.org/"
    WriteRegDWORD HKLM "${DELUGE_UNINSTALL_KEY}" "NoModify" 1
    WriteRegDWORD HKLM "${DELUGE_UNINSTALL_KEY}" "NoRepair" 1
    WriteRegStr HKLM "${DELUGE_UNINSTALL_KEY}" "UninstallString" "$INSTDIR\${DELUGE_UNINST_EXE}"
    ; Sets scope of the desktop and Start Menu entries for all users.
    SetShellVarContext "all"
    Goto deluge_install_files

  deluge_hkcu:
    WriteRegStr HKCU ${DELUGE_REG_KEY} "" "$INSTDIR"
    WriteRegStr HKCU ${DELUGE_REG_KEY} "Version" "${DELUGE_VERSION}"
    WriteRegStr HKCU "${DELUGE_UNINSTALL_KEY}" "DisplayName" "Deluge"
    WriteRegStr HKCU "${DELUGE_UNINSTALL_KEY}" "DisplayVersion" "${DELUGE_VERSION}"
    WriteRegStr HKCU "${DELUGE_UNINSTALL_KEY}" "HelpLink" "http://deluge-torrent.org/"
    WriteRegDWORD HKCU "${DELUGE_UNINSTALL_KEY}" "NoModify" 1
    WriteRegDWORD HKCU "${DELUGE_UNINSTALL_KEY}" "NoRepair" 1
    WriteRegStr HKCU "${DELUGE_UNINSTALL_KEY}" "UninstallString" "$INSTDIR\${DELUGE_UNINST_EXE}"
    Goto deluge_install_files

  deluge_none:

  deluge_install_files:
    SetOutPath "$INSTDIR"
    ; Deluge files
    SetOverwrite on

    File /r .\dist\*.*
    !ifdef DEBUG
    ;File "${DELUGE_INSTALLER_DEPS}\exchndl.dll"
    !endif

    ; Install shfolder.dll if need be..
    ;SearchPath $R4 "shfolder.dll"
    ;StrCmp $R4 "" 0 got_shfolder
    ;  SetOutPath "$SYSDIR"
    ;  File "${DELUGE_INSTALLER_DEPS}\shfolder.dll"
    ;  SetOutPath "$INSTDIR"
    ;got_shfolder:

    SetOutPath "$INSTDIR"

    ; If we don't have install rights we're done
    StrCmp $R0 "NONE" done
    SetOverwrite off

    ; write out uninstaller
    SetOverwrite on
    WriteUninstaller "$INSTDIR\${DELUGE_UNINST_EXE}"
    SetOverwrite off

    ; If we previously had deluge set up to run on startup, make it do so again
    StrCmp $STARTUP_RUN_KEY "HKCU" +1 +2
    WriteRegStr HKCU "${STARTUP_RUN_KEY}" "Deluge" "$INSTDIR\deluge.exe"
    StrCmp $STARTUP_RUN_KEY "HKLM" +1 +2
    WriteRegStr HKLM "${STARTUP_RUN_KEY}" "Deluge" "$INSTDIR\deluge.exe"

  done:
SectionEnd ; end of default Deluge section

;--------------------------------
;Shortcuts

SectionGroup /e $(DELUGE_SHORTCUTS_SECTION_TITLE) SecShortcuts
  Section /o $(DELUGE_DESKTOP_SHORTCUT_SECTION_TITLE) SecDesktopShortcut
    SetOverwrite on
    CreateShortCut "$DESKTOP\Deluge.lnk" "$INSTDIR\deluge.exe"
    SetOverwrite off
  SectionEnd
  Section $(DELUGE_STARTMENU_SHORTCUT_SECTION_TITLE) SecStartMenuShortcut
    SetOverwrite on
    CreateShortCut "$SMPROGRAMS\Deluge.lnk" "$INSTDIR\deluge.exe"
    SetOverwrite off
  SectionEnd
SectionGroupEnd

SectionGroup /e $(DELUGE_WINDOWS_SETTINGS_TITLE) SecWindows
  Section $(DELUGE_ASSOCIATE_TORRENT_TITLE) SecAssociateTorrents
	!insertmacro APP_ASSOCIATE "torrent" "DelugeTorrent.TorrentFile" "Torrent File" "$INSTDIR\deluge.exe,0" "Open with deluge" "$\"$INSTDIR\deluge.exe$\" $\"%1$\""
  SectionEnd
  Section /o $(DELUGE_WINDOWS_START_TITLE) SecStartDelugeWithWindows
	WriteRegStr HKCU "${STARTUP_RUN_KEY}" "Deluge" "$INSTDIR\deluge.exe"
  SectionEnd
SectionGroupEnd

;--------------------------------
;Uninstaller Section


Section Uninstall
  Call un.CheckUserInstallRights
  Pop $R0
  StrCmp $R0 "NONE" no_rights
  StrCmp $R0 "HKCU" try_hkcu try_hklm

  try_hkcu:
    ReadRegStr $R0 HKCU ${DELUGE_REG_KEY} ""
    StrCmp $R0 $INSTDIR 0 cant_uninstall
      ; HKCU install path matches our INSTDIR so uninstall
      DeleteRegKey HKCU ${DELUGE_REG_KEY}
      DeleteRegKey HKCU "${DELUGE_UNINSTALL_KEY}"
      Goto cont_uninstall

  try_hklm:
    ReadRegStr $R0 HKLM ${DELUGE_REG_KEY} ""
    StrCmp $R0 $INSTDIR 0 try_hkcu
      ; HKLM install path matches our INSTDIR so uninstall
      DeleteRegKey HKLM ${DELUGE_REG_KEY}
      DeleteRegKey HKLM "${DELUGE_UNINSTALL_KEY}"
      DeleteRegKey HKLM "${HKLM_APP_PATHS_KEY}"
      ; Sets start menu and desktop scope to all users..
      SetShellVarContext "all"

  cont_uninstall:
    ; The WinPrefs plugin may have left this behind..
    DeleteRegValue HKCU "${STARTUP_RUN_KEY}" "Deluge"
    DeleteRegValue HKLM "${STARTUP_RUN_KEY}" "Deluge"

    ; Remove Language preference info (TODO: check if NSIS removes this)
    !insertmacro APP_UNASSOCIATE "torrent" "DelugeTorrent.TorrentFile"

    RMDir /r "$INSTDIR\share"
    RMDir /r "$INSTDIR\lib"
    RMDir /r "$INSTDIR\etc"
    RMDir /r "$INSTDIR\dbus-daemon"
    Delete "$INSTDIR\*.pyd"
    Delete "$INSTDIR\*.dll"
    Delete "$INSTDIR\dbus-daemon.exe"
    Delete "$INSTDIR\w9xpopen.exe"
    Delete "$INSTDIR\deluge.exe"
    Delete "$INSTDIR\${DELUGE_UNINST_EXE}"
    Delete "$INSTDIR\install.log"

    ;Try to remove Deluge install dir (only if empty)
    RMDir "$INSTDIR"

    ; Shortcuts..
    Delete "$DESKTOP\Deluge.lnk"

    Goto done

  cant_uninstall:
    MessageBox MB_OK $(un.DELUGE_UNINSTALL_ERROR_1) /SD IDOK
    Quit

  no_rights:
    MessageBox MB_OK $(un.DELUGE_UNINSTALL_ERROR_2) /SD IDOK
    Quit

  done:
SectionEnd ; end of uninstall section

;--------------------------------
;Descriptions
!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
  !insertmacro MUI_DESCRIPTION_TEXT ${SecDeluge} \
        $(DELUGE_SECTION_DESCRIPTION)

  !insertmacro MUI_DESCRIPTION_TEXT ${SecWindows} \
	$(DELUGE_WINDOWS_SETTINGS_DESC)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecAssociateTorrents} \
	$(DELUGE_ASSOCIATE_TORRENT_DESC)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecStartDelugeWithWindows} \
	$(DELUGE_WINDOWS_START_DESC)

  !insertmacro MUI_DESCRIPTION_TEXT ${SecShortcuts} \
        $(DELUGE_SHORTCUTS_SECTION_DESCRIPTION)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecDesktopShortcut} \
        $(DELUGE_DESKTOP_SHORTCUT_DESC)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecStartMenuShortcut} \
        $(DELUGE_STARTMENU_SHORTCUT_DESC)

!insertmacro MUI_FUNCTION_DESCRIPTION_END

;--------------------------------
;Functions

!macro CheckUserInstallRightsMacro UN
Function ${UN}CheckUserInstallRights
  Push $0
  Push $1
  ClearErrors
  UserInfo::GetName
  IfErrors Win9x
  Pop $0
  UserInfo::GetAccountType
  Pop $1

  StrCmp $1 "Admin" 0 +3
    StrCpy $1 "HKLM"
    Goto done
  StrCmp $1 "Power" 0 +3
    StrCpy $1 "HKLM"
    Goto done
  StrCmp $1 "User" 0 +3
    StrCpy $1 "HKCU"
    Goto done
  StrCmp $1 "Guest" 0 +3
    StrCpy $1 "NONE"
    Goto done
  ; Unknown error
  StrCpy $1 "NONE"
  Goto done

  Win9x:
    StrCpy $1 "HKLM"

  done:
    Exch $1
    Exch
    Pop $0
FunctionEnd
!macroend
!insertmacro CheckUserInstallRightsMacro ""
!insertmacro CheckUserInstallRightsMacro "un."

;
; Usage:
;   Push $0 ; Path string
;   Call VerifyDir
;   Pop $0 ; 0 - Bad path  1 - Good path
;
Function VerifyDir
  Exch $0
  Push $1
  Push $2
  Loop:
    IfFileExists $0 dir_exists
    StrCpy $1 $0 ; save last
    ${GetParent} $0 $0
    StrLen $2 $0
    ; IfFileExists "C:" on xp returns true and on win2k returns false
    ; So we're done in such a case..
    IntCmp $2 2 loop_done
    ; GetParent of "C:" returns ""
    IntCmp $2 0 loop_done
    Goto Loop

  loop_done:
    StrCpy $1 "$0\GaImFooB"
    ; Check if we can create dir on this drive..
    ClearErrors
    CreateDirectory $1
    IfErrors DirBad DirGood

  dir_exists:
    ClearErrors
    FileOpen $1 "$0\delugefoo.bar" w
    IfErrors PathBad PathGood

    DirGood:
      RMDir $1
      Goto PathGood1

    DirBad:
      RMDir $1
      Goto PathBad1

    PathBad:
      FileClose $1
      Delete "$0\delugefoo.bar"
      PathBad1:
      StrCpy $0 "0"
      Push $0
      Goto done

    PathGood:
      FileClose $1
      Delete "$0\delugefoo.bar"
      PathGood1:
      StrCpy $0 "1"
      Push $0

  done:
  Exch 3 ; The top of the stack contains the output variable
  Pop $0
  Pop $2
  Pop $1
FunctionEnd

Function .onVerifyInstDir
  Push $0
  Push $INSTDIR
  Call VerifyDir
  Pop $0
  StrCmp $0 "0" 0 dir_good
  Pop $0
  Abort

  dir_good:
  Pop $0
FunctionEnd

!macro RunCheckMacro UN
Function ${UN}RunCheck
    FindWindow $0 "gdkWindowTopLevel" "Deluge"
    IsWindow $0 0 done 
    MessageBox MB_OK|MB_ICONEXCLAMATION $(DELUGE_IS_RUNNING) /SD IDOK
    Abort
  done:
FunctionEnd
!macroend
!insertmacro RunCheckMacro ""
!insertmacro RunCheckMacro "un."

Function .onInit
  Push $R0
  System::Call 'kernel32::CreateMutexA(i 0, i 0, t "deluge_installer_running") i .r1 ?e'
  Pop $R0
  StrCmp $R0 0 +3
    MessageBox MB_OK|MB_ICONEXCLAMATION $(INSTALLER_IS_RUNNING) /SD IDOK
    Abort
  Call RunCheck
  StrCpy $name "Deluge ${DELUGE_VERSION}"

  ;Preselect the URI handlers as appropriate
  ;Call SelectURIHandlerSelections

  ;Preselect the "shortcuts" checkboxes according to the previous installation
  ClearErrors
  ;Make sure that there was a previous installation
  ReadRegStr $R0 HKCU "${DELUGE_REG_KEY}" "Installer Language"
  IfErrors done_preselecting_shortcuts
    ;Does the Desktop shortcut exist?
    GetFileTime "$DESKTOP\Deluge.lnk" $R0 $R0
    IfErrors +1 +5
    ClearErrors
    SetShellVarContext "all"
    GetFileTime "$DESKTOP\Deluge.lnk" $R0 $R0
    IfErrors preselect_startmenu_shortcut ;Desktop Shortcut if off by default
    !insertmacro SelectSection ${SecDesktopShortcut}
  preselect_startmenu_shortcut:
    ;Reset ShellVarContext because we may have changed it
    SetShellVarContext "current"
    ClearErrors
    ;Does the StartMenu shortcut exist?
    GetFileTime "$SMPROGRAMS\Deluge.lnk" $R0 $R0
    IfErrors +1 done_preselecting_shortcuts ;StartMenu Shortcut is on by default
    ClearErrors
    SetShellVarContext "all"
    GetFileTime "$SMPROGRAMS\Deluge.lnk" $R0 $R0
    IfErrors +1 done_preselecting_shortcuts ;StartMenu Shortcut is on by default
    !insertmacro UnselectSection ${SecStartMenuShortcut}
  done_preselecting_shortcuts:
  ;Reset ShellVarContext because we may have changed it
  SetShellVarContext "current"

  StrCpy $ISSILENT "/NOUI"

  ${GetParameters} $R0
  ClearErrors
  ${GetOptions} $R0 "/L=" $R0
  IfErrors +3
  StrCpy $LANGUAGE $R0
  Goto skip_lang

  ; Select Language
    ; Display Language selection dialog
    !insertmacro MUI_LANGDLL_DISPLAY
    skip_lang:

  ; If install path was set on the command, use it.
  StrCmp $INSTDIR "" 0 instdir_done

  ;  If deluge or gaim is currently installed, we should default to where it is currently installed
  ClearErrors
  ReadRegStr $INSTDIR HKCU "${DELUGE_REG_KEY}" ""
  IfErrors +2
  StrCmp $INSTDIR "" 0 instdir_done
  ClearErrors
  ReadRegStr $INSTDIR HKLM "${DELUGE_REG_KEY}" ""
  IfErrors +2
  StrCmp $INSTDIR "" 0 instdir_done

  Call CheckUserInstallRights
  Pop $R0

  StrCmp $R0 "HKLM" 0 user_dir
    StrCpy $INSTDIR "$PROGRAMFILES\Deluge"
    Goto instdir_done
  user_dir:
    Push $SMPROGRAMS
    ${GetParent} $SMPROGRAMS $R2
    ${GetParent} $R2 $R2
    StrCpy $INSTDIR "$R2\Deluge"

  instdir_done:
;LogSet on
  Pop $R0
FunctionEnd

Function un.onInit
  Call un.RunCheck
  StrCpy $name "Deluge ${DELUGE_VERSION}"

  ; Get stored language preference
  !insertmacro MUI_UNGETLANGUAGE

FunctionEnd

; This is a modified StartRadioButtons (from Sections.nsh)
; The only difference is that it allows for nothing in the group to be selected
; In that case, the default variable should be set to ""
!macro StartRadioButtonsUnselectable var

  !define StartRadioButtons_Var "${var}"

  Push $R0
  Push $R1

   ;If we have no selection, don't try to unselect it
   StrCmp "${StartRadioButtons_Var}" "" +4
   SectionGetFlags "${StartRadioButtons_Var}" $R0
   IntOp $R1 $R0 & ${SF_SELECTED}
   IntOp $R0 $R0 & ${SECTION_OFF}
   SectionSetFlags "${StartRadioButtons_Var}" $R0

   ; If the previous value isn't currently selected,
   ; we don't want to select it at the end
   IntCmp $R1 ${SF_SELECTED} +2
   StrCpy "${StartRadioButtons_Var}" ""

   StrCpy $R1 "${StartRadioButtons_Var}"

!macroend
